@@ -1,30 +1,40 @@
 #!/usr/bin/python2
 
 import sys
-
-import curses
-#FIXME: add checking for use_default_colors() and curses_extra
-import curses_extra
-import curses.textpad
-
 import os, shutil
-
+import curses
+import curses.textpad
 import urllib
 import timeoutsocket
-
 import locale
-
 import time
-
 import string, StringIO
 import re
 import sha
+import textwrap
 
+# 3rd party modules
 import feedparser
-
 import html2text
 
-import textwrap
+
+# use_default_colors was added in python 2.4
+# if we don't have it, don't bother with colors
+if not hasattr(curses, "use_default_colors"):
+    try:
+        import curses_extra
+        curses.use_default_colors = curses_extra.use_default_colors
+        BGCOLOR = -1
+        USE_COLORS = 1
+    except:
+        def use_default_colors():
+            pass
+        BGCOLOR = curses.COLOR_WHITE
+        USE_COLORS = 0
+        curses.use_default_colors = use_default_colors
+else:
+    BGCOLOR = -1
+    USE_COLORS = 1
 
 reload(sys)
 sys.setdefaultencoding('utf-8')
@@ -35,7 +45,7 @@ timeoutsocket.setDefaultSocketTimeout(20)
 
 BROWSER = r"lynx %s"
 COLOR = {}
-VERSION = "0.9.2"
+VERSION = "0.9.3"
 RCDIR = "%s/.goodnews-%s" % (os.getenv("HOME"), VERSION)
 
 OS = sys.platform
@@ -48,19 +58,29 @@ class GoodnewsURLopener(urllib.FancyURLopener):
     version = USER_AGENT
 urllib._urlopener = GoodnewsURLopener()
 
-USE_COLORS = 0
+
 CMD_SLOTS_COLUMNS =6 
 CMD_SLOTS = (CMD_SLOTS_COLUMNS * 2) - 1
 
 SHOW_BOZO = 0
+CURSOR_ON = 1
 
 cur_cmd_grp = []
 keybinding = {}
 first_ptr = []
+first_scr_ptr = 0
 
+class FeedMenu:
+
+    def __init__(self):
+        self.first_scr_ptr = 0
+        self.items = {}
+        self.sorted = []
+        
 
 class Feed:
     def __init__(self):
+        self.id = ""
         self.feedurl = ""
         self.feed = ""
         self.feed_data = ""
@@ -116,11 +136,11 @@ def ex_entry_field(stdscr, default=""):
         text_win.addstr(0, 0, default)
     
     #curses.echo()
-    curses.curs_set(1)
+    #curses.curs_set(1)
     
     t = textpad.edit()
 
-    curses.curs_set(0)
+    #curses.curs_set(0)
     
     return t
     
@@ -135,25 +155,27 @@ def InitCurses():
 
     stdscr.clear()
 
-    curses.curs_set(0)
+    
     stdscr.refresh()
     LINES, COLS = stdscr.getmaxyx()
 
-    curses.start_color();
+    if USE_COLORS:
+        
+        curses.start_color();
 
-    #default color for fg/bg set to -1
-    curses_extra.use_default_colors()
+        #default color for fg/bg set to -1
+        curses.use_default_colors()
 
-    # FIXME: find a pleasing color combo
-    curses.init_pair (10, 1, -1); #/* red */
-    curses.init_pair (11, 2, -1); #/* green */
-    curses.init_pair (12, 3, -1); #/* orange */
-    curses.init_pair (13, 4, -1); #/* blue */
-    curses.init_pair (14, 5, -1); #/* magenta */
-    curses.init_pair (15, 6, -1); #/* cyan */
-    curses.init_pair (16, 7, -1); #* gray */
-    curses.init_pair (17, curses.COLOR_YELLOW, curses.COLOR_BLUE)
-    curses.init_pair (18, curses.COLOR_WHITE, curses.COLOR_BLUE)
+        # FIXME: find a pleasing color combo
+        curses.init_pair (10, 1, BGCOLOR); #/* red */
+        curses.init_pair (11, 2, BGCOLOR); #/* green */
+        curses.init_pair (12, 3, BGCOLOR); #/* orange */
+        curses.init_pair (13, 4, BGCOLOR); #/* blue */
+        curses.init_pair (14, 5, BGCOLOR); #/* magenta */
+        curses.init_pair (15, 6, BGCOLOR); #/* cyan */
+        curses.init_pair (16, 7, BGCOLOR); #* gray */
+        curses.init_pair (17, curses.COLOR_YELLOW, curses.COLOR_BLUE)
+        curses.init_pair (18, curses.COLOR_WHITE, curses.COLOR_BLUE)
 
     return stdscr
     
@@ -165,9 +187,12 @@ def draw_status(stdscr, txt, delay=0):
 
     # clear
     stdscr.addstr(LINES-3, 0, ' '*COLS)
-        
-    stdscr.addnstr (LINES-3, 1, txt, COLS-2, curses.color_pair(10));
-	
+
+    if USE_COLORS:
+        stdscr.addnstr (LINES-3, 1, txt, COLS-2, curses.color_pair(10));
+    else:
+        stdscr.addnstr (LINES-3, 1, txt, COLS-2);
+
     stdscr.attroff (curses.A_BOLD);
     stdscr.refresh()
     
@@ -177,7 +202,7 @@ def draw_status(stdscr, txt, delay=0):
 
 
 
-def draw_header (stdscr, filterstring="", use_colors=1):
+def draw_header (stdscr, filterstring="", use_colors=USE_COLORS):
     LINES, COLS = stdscr.getmaxyx()
     #stdscr.attron (curses.A_STANDOUT)
 
@@ -245,6 +270,7 @@ def draw_footer(stdscr, cur_grp, bindings):
         
             stdscr.addstr(y, (x * slotspace) + len(repr(i)) + 1, outstr)
 
+
         
 
 def dump_traceback(stdscr, s):
@@ -292,12 +318,16 @@ def keybind_morecmds(stdscr, **args):
     draw_footer(stdscr, cur_cmd_grp, bindings)
 
 
+def keybind_cursor(stdscr, **args):
+    global CURSOR_ON
 
-#def keybind_next(stdscr, **args):
-#    pass
+    CURSOR_ON = not CURSOR_ON
+
+    if CURSOR_ON:
+        curses.curs_set(0)
+    else:
+        curses.curs_set(1)
 
-#def keybind_prev (stdscr, **args):
-#    pass
 
 def keybind_bozo(stdscr, **args):
     global SHOW_BOZO
@@ -316,7 +346,6 @@ def keybind_addfeed(stdscr, **args):
         highlighted = first_ptr[-1]
     
         
-
 def keybind_delfeed(stdscr, **args):
 
     deletion = highlighted
@@ -369,7 +398,6 @@ def keybind_reload(stdscr, **args):
     update_feed(stdscr, highlighted)
 
 def keybind_reloadall(stdscr, **args):
-    #draw_status(stdscr, repr(args['bindings']), 4)
     update_all_feeds(stdscr, first_ptr, draw_feeds, highlighted, args['bindings'])
 
 
@@ -425,6 +453,24 @@ def keybind_next(stdscr, **args):
     if first_ptr.index(highlighted) < len(first_ptr) -1:
         highlighted = first_ptr[first_ptr.index(highlighted) + 1]
 
+
+# new test
+def keybind_next(stdscr, **args):
+    global highlighted, first_scr_ptr
+
+    LINES,COLS = stdscr.getmaxyx()
+    maxlines = LINES - 6
+
+    if not first_ptr:
+        return
+
+    if first_ptr.index(highlighted) < len(first_ptr) -1:
+        highlighted = first_ptr[first_ptr.index(highlighted) + 1]
+
+    if first_ptr.index(highlighted) - first_scr_ptr > (maxlines - 4):
+        first_scr_ptr += 1
+
+
 def keybind_prev(stdscr, **args):
     global highlighted
 
@@ -434,6 +480,20 @@ def keybind_prev(stdscr, **args):
     if first_ptr.index(highlighted) > 0:
         highlighted = first_ptr[first_ptr.index(highlighted) - 1]
 
+# new test
+def keybind_prev(stdscr, **args):
+    global highlighted, first_scr_ptr
+    
+    if not first_ptr:
+        return
+    
+    if first_ptr.index(highlighted) > 0:
+        highlighted = first_ptr[first_ptr.index(highlighted) - 1]
+
+    if (first_ptr.index(highlighted) - first_scr_ptr) < 0:
+        first_scr_ptr -= 1
+    
+
 def keybind_feed_down(stdscr, **args):
     LINES,COLS = stdscr.getmaxyx()
     maxlines = LINES - 6
@@ -533,7 +593,6 @@ def setup_cmd_grp(bindings):
             cmd_grp.append(j)
             
 
-    #raise "err", len(cmd_grp
     if len(cmd_grp) > CMD_SLOTS:
         grp = cmd_grp[:CMD_SLOTS]
         grp.append('^O')
@@ -543,10 +602,6 @@ def setup_cmd_grp(bindings):
     return grp
 
 
-#def keybind_feed_next_page(stdscr, **args):
-#    cur_feed.highlighted.page_showed += 1
-    
-    
 def keybind_feed_enter_quit(stdscr, **args):
     while cur_feed.highlighted.id != cur_feed.sorted[-1]:
         keybind_feed_down(stdscr)
@@ -576,7 +631,6 @@ def keybind_feed_select(stdscr, **args):
     item_keybind['B'] = 'keybind_browser'
     
     cur_cmd_grp = setup_cmd_grp(item_keybind)
-    #cur_keybind = item_keybind
     
     while 1:
         draw_item(stdscr, cur_feed)
@@ -590,6 +644,7 @@ def do_item_loop(stdscr, item, keybindings):
     while 1:
         stdscr.getch()
 
+
 def enter_main(stdscr, ptr, keybindings, encoding='latin-1'):
     global highlighted, first_ptr, saved_keybindings, syscall, cur_cmd_grp
 
@@ -681,10 +736,6 @@ def pretty_print_item(stdscr, description):
 
 def pretty_item_lines(description):
 
-    #LINES, COLS = stdscr.getmaxyx()
-    #columns = COLS - 10
-
-    
     tp = []
     for n in description.split("\n"):
         tp.append(n.strip())
@@ -705,7 +756,6 @@ def pretty_item_lines(description):
             line = "-"*columns
         outlines.append(line)
 
-    
     return outlines
 
   
@@ -715,9 +765,6 @@ def test_pretty_print_item(description):
     columns = COLS - 10
     SNIP = LINES - 13
 
-    #paragraphs = description.split("\n  \n")
-    #paragraphs = description.split("\n\n")
-
     tp = []
     for n in description.split("\n"):
         tp.append(n.strip())
@@ -736,7 +783,6 @@ def test_pretty_print_item(description):
         outlines[SNIP - 1] = outlines[SNIP -1][:-3] + '...'
     
     for line in outlines[:SNIP]:
-       # print repr(line)
         if line == '* * *':
             line = "-"*columns
             
@@ -760,7 +806,7 @@ def draw_item(stdscr, feed, use_colors=1, encoding='latin-1'):
 
     if feed.description:
         des = feed.description.encode(encoding, 'replace')
-        #des = feed.description
+
         if len(des) > columns:
             des = des[:columns - 5 - 3] + '...'
 
@@ -770,12 +816,9 @@ def draw_item(stdscr, feed, use_colors=1, encoding='latin-1'):
     stdscr.move(5,4)
     stdscr.move(5,0)
     pretty_print_item(stdscr, item.title.encode(encoding, 'replace'))
-    #pretty_print_item(stdscr, item.title)
-    #stdscr.addstr('\n\n    --\n    ')
     stdscr.addstr('\n   --\n')
 
     pretty_print_item(stdscr, item.description.encode(encoding, 'replace'))
-    #pretty_print_item(stdscr, item.description)
 
     draw_link(stdscr, item)
     
@@ -803,7 +846,7 @@ def draw_link(stdscr, item):
 
 
 
-def draw_news(stdscr, feed, use_colors=1, encoding='latin-1'):
+def draw_news(stdscr, feed, use_colors=USE_COLORS, encoding='latin-1'):
     global cur_feed
     
     ypos = 4
@@ -826,7 +869,7 @@ def draw_news(stdscr, feed, use_colors=1, encoding='latin-1'):
 
     if feed.description:
         des = feed.description.encode(encoding, 'replace')
-        #des = feed.description
+
         if len(des) > columns:
             des = des[:columns-5-3] + '...'
         stdscr.addstr(2, 1, des, curses.A_BOLD)
@@ -838,7 +881,6 @@ def draw_news(stdscr, feed, use_colors=1, encoding='latin-1'):
 
         if item.title:
             title = item.title.encode(encoding, 'replace')
-            #title = item.title
         else:
             title = item.link
 
@@ -847,7 +889,6 @@ def draw_news(stdscr, feed, use_colors=1, encoding='latin-1'):
             curses.A_STANDOUT = curses.A_STANDOUT|curses.A_BOLD
 
         if item == feed.highlighted:
-            #syscall = doupdatelink(stdscr, item.link)
             draw_link(stdscr, item)
             
             stdscr.attron(curses.A_STANDOUT)
@@ -860,7 +901,9 @@ def draw_news(stdscr, feed, use_colors=1, encoding='latin-1'):
 
         if not item.readstatus:
             stdscr.attroff(curses.A_BOLD)
-            curses.A_STANDOUT = curses.color_pair(18)
+
+            if use_colors:
+                curses.A_STANDOUT = curses.color_pair(18)
             
         if item == feed.highlighted:
             stdscr.attroff(curses.A_STANDOUT)
@@ -870,13 +913,13 @@ def draw_news(stdscr, feed, use_colors=1, encoding='latin-1'):
             break
 
         
-def draw_feeds(stdscr, feeds, highlighted='', use_colors=1, keybindings={}):
+def draw_feeds(stdscr, feeds, highlighted='', keybindings={}):
     ypos = 2
     LINES, COLS = stdscr.getmaxyx()
     columns = COLS - 10
     maxlines = LINES - 6
 
-    if use_colors:
+    if USE_COLORS:
         curses.A_STANDOUT = curses.color_pair(18)
 
     if not feeds:
@@ -889,7 +932,7 @@ def draw_feeds(stdscr, feeds, highlighted='', use_colors=1, keybindings={}):
         stdscr.addstr(10, center - cos, outstr)
         return
         
-    for feed in feeds:
+    for feed in feeds[first_scr_ptr:]:
         stdscr.move(ypos, 0)
         stdscr.clrtoeol()
         newcount = 0
@@ -929,7 +972,8 @@ def draw_feeds(stdscr, feeds, highlighted='', use_colors=1, keybindings={}):
             stdscr.attroff(curses.A_STANDOUT)
 
         ypos += 1
-
+        if ypos > maxlines:
+            break
         
 
 def doupdatelink(stdscr, link):
@@ -1056,13 +1100,9 @@ class FeedParser:
 
             if link:
                 new_item = NewsItem()
-                #new_item.title = replace_qoutes(strip_tags(title))
                 new_item.title = html2text.html2text(title)
-                #new_item.title = strip_tags(title)
                 new_item.link  = link
-                #new_item.description = replace_qoutes(strip_tags(description))
                 new_item.description = html2text.html2text(strip_links(description))
-                #new_item.description = strip_tags(description)
                 new_item.id = id
             else:
                 return
@@ -1112,8 +1152,6 @@ def replace_qoutes(txt):
     qoutes = re.compile('&.*?;', re.S)
     
     ents = get_unicode_entitydefs()
-    #ents = htmlentitydefs.entitydefs
-    
     all_qoutes = qoutes.findall(txt)
     
     for q in all_qoutes:
@@ -1143,6 +1181,10 @@ def strip_tags(txt):
     return tags.sub('', txt)
 
 def strip_links(txt):
+
+    if not txt:
+        return ''
+    
     link = re.compile('</?a.*?>', re.S|re.I)
     img = re.compile('</?img.*?>', re.S|re.I)    
     br = re.compile('<br.*?>', re.S|re.I)
@@ -1278,34 +1320,6 @@ def load_state_config(stdscr):
         except IOError, err:
             clean_up_and_exit ('Create initial configfile "urls', err); # /* Still didn't work? */
 
-
-	
-    """
-    keybind['next'] = 'n'
-    keybind['prev'] = 'p'
-    keybind['quit'] = 'q'
-    keybind['addfeed'] = 'a'
-    keybind['deletefeed'] = 'D'
-    keybind['allread'] = 'm'
-    keybind['dfltbrowser'] = 'B'
-    keybind['moveup'] = 'P'
-    keybind['movedown'] = 'N'
-    keybind['feedinfo'] = 'i'
-    keybind['reload'] = 'r'
-    keybind['reloadall'] = 'R'
-    keybind['urljump'] = 'o'
-    keybind['urljump2'] = 'O'
-    keybind['changefeedname'] = 'c'
-    keybind['sortfeeds'] = 's'
-    keybind['pup'] = 'b'
-    keybind['pdown'] = ' '
-    keybind['categorize'] = 'C'
-    keybind['filter'] = 'f'
-    keybind['filtercurrent'] = 'g'
-    keybind['nofilter'] = 'F'
-    keybind['help'] = 'h'
-    keybind['about'] = 'A'
-    """
     
     # defaults keybindings 
 
@@ -1329,7 +1343,7 @@ def load_state_config(stdscr):
     #keybinding['b'] = 'keybind_pup'
     #keybinding[' '] = 'keybind_pdown'
     keybinding[' '] = 'keybind_select'    
-    #keybinding['C'] = 'keybind_categorize'
+    #keybinding['C'] = 'keybind_cursor'
     #keybinding['f'] = 'keybind_filter'
     #keybinding['g'] = 'keybind_filtercurrent'
     #keybinding['F'] = 'keybind_nofilter'
@@ -1356,34 +1370,7 @@ def load_state_config(stdscr):
     #keybinding['^O'] = 'keybind_morecmds'
     
 
-    """
-    fname = RCDIR + "/keybindings"
-
-    try:
-        #configfile = open(fname)
-        for line in open(fname).readlines():
-            if line[0] == "#":
-                continue
-            
-            keyb, key = line.strip().split(':')
-
-            if keytrans.has_key(keyb):
-                keybinding[key] = "keybind_" + keytrans[keyb]
-
-        if keybinding['b'] == 'keybind_browser':
-            keybinding['B'] = 'keybind_browser'
-            del(keybinding['b'])
-            
-    except IOError:
-
-        #/* Create new file. */
-        configfile = open (fname, "w+")
-
-        configfile.write(default_keybindings)
-        configfile.close()
-
-        #here 346
-    """
+    # FIXME: read keybindings from file here
 	
     #/***********************
     #* COLOR suppport code *
@@ -1394,12 +1381,10 @@ def load_state_config(stdscr):
     COLOR["newitemsbold"] = 0
     COLOR["urljump"] = 4
     COLOR["urljumpbold"] = 0
-	
-    use_colors = 1
+    
+    use_colors = USE_COLORS
     
     fname = RCDIR + "/colors"
-
-
     try:
         #configfile = open(fname)
         for line in open(fname).readline():
@@ -1419,48 +1404,13 @@ def load_state_config(stdscr):
                 COLOR['urljump'] = int(line[1])
                 if COLOR['urljump'] > 7:
                     COLOR['urljumlbold'] = 1
-
     except IOError:
-	
         #/* Set color configfile update flag. */
         COLOR["changed"] = 1
 
-    """
-    if use_colors: 
-        curses.start_color();
-		
-        #/* The following call will automagically implement -1 as the terminal's
-        #default color for fg/bg in init_pair. */
-        curses_extra.use_default_colors()
-		
-        #/* Internally used color pairs.
-        #Use with WA_BOLD to get bright color (orange->yellow, gray->white, etc). */
-        curses.init_pair (10, 1, 0); #/* red */
-        curses.init_pair (11, 2, -1); #/* green */
-        curses.init_pair (12, 3, -1); #/* orange */
-        curses.init_pair (13, 4, -1); #/* blue */
-        curses.init_pair (14, 5, -1); #/* magenta */
-        curses.init_pair (15, 6, -1); #/* cyan */
-        curses.init_pair (16, 7, -1); #* gray */
-		
-        #/* Default terminal color color pair */
-        curses.init_pair (63, -1, -1);
-		
-        #/* Initialize all color pairs we're gonna use. */
-        #/* New item. */
-        #if COLOR["newitemsbold"]:
-        #    curses.init_pair (2, COLOR["newitems"]-8, -1)
-        #else:
-        #curses.init_pair (1, curses.COLOR_MAGENTA, curses.COLOR_WHITE)
-	#		
-        #/* Goto url line */
-        #if COLOR["urljumpbold"]:
-        #    curses.init_pair (3, COLOR["urljump"]-8, -1)
-        #else:
-        #    curses.init_pair (3, COLOR["urljump"], -1)
-	
-    """
 
+    #FIXME: read color config from file here
+    
 
 # Update feed from server.
 def update_feed(stdscr, cur_ptr):
@@ -1552,13 +1502,6 @@ def load_feed(stdscr,cur_ptr):
     if cur_ptr.bozo:
         draw_status(stdscr, "Warning: bozo XML! Maybe not a RSS feed?", 2)
 
-    """
-    if xml.error:
-        draw_status (stdscr, "Invalid XML: '%s'"% xml.error,2)    
-        draw_status (stdscr, "Invalid XML! Cannot parse '%s'"% fname,2)
-        first_ptr.remove(cur_ptr)
-        return 0
-    """
     
     # get readstatus
     fname = RCDIR + "/cache-readstatus/%s" % (hashme)
@@ -1738,6 +1681,8 @@ def add_feed(stdscr, first_ptr):
     return 1
 
 
+
+
 if __name__ == '__main__':
 
     #FIXME: argv checking
